% This code shows wavefront(wf) reconstruction of sample induced aberration
% and full AO correction

%% load system aberration and calibrated basis
%load Z2C matrix
load('../Z2C.mat/');
% load DM command for system aberration correction 
load('../DMcommand_system_aberration.mat/');
% load DM-SHWS calibration basis
load('../shstruct.mat/');

%% calculate the wf slope of sample induced aberration
% load reference wavefront spots image measure from fluorescent solution bath
sample_ref=int16(imread('reference_wf.tif')); 
% calculate the centres of the reference wavefront slope
ref_centres_property = shwfs_get_centres(sample_ref, shstruct, shstruct.centroid); 
% get the x and y shifts of the spots from reference wavefront
slope_sampleref =ref_centres_property(:,1:2);

% load the wavefront spots image measure from the imaging sample
sample=int16(imread(sprintf('%s.tif',experiment_index))); 
% calculate the centres of the reference wavefront spots
sample_centres_property = shwfs_get_centres(sample, shstruct, shstruct.centroid); 
% get the x and y shifts of the spots from sample induced aberrated wavefront
slope_sample = sample_centres_property(:,1:2);
% get the SBR weight information of each spots
slope_weight_sample = sample_centres_property(:,3);

% scale the weight into 0 to 1
SBRmin=1.2;
SBRmax=2;
slope_weight_sample(slope_weight_sample <= SBRmin)=0;
slope_weight_sample(slope_weight_sample >= SBRmax)=1;
slope_weight_index = (slope_weight_sample>SBRmin)&(slope_weight_sample<SBRmax);
slope_weight_sample(slope_weight_index)= (slope_weight_sample(slope_weight_index)-SBRmin)/(SBRmax-SBRmin);
slope_weight_matrix = diag([slope_weight_sample;slope_weight_sample]);

% calculate the relative slope between reference and sample induced to
% eliminate the effects from system aberration
deltas = slope_sample - slope_sampleref;

% wf reconsruction to zernike mode 
nspots=shstruct.nspots;
sample_aberration = reshape(deltas,nspots*2,1);
% non-weighted reconstruction
DM_zernike_index = shstruct.Geninv*sample_aberration;
% weighted reconstruction
DM_zernike_index_weight = pinv(slope_weight_matrix*shstruct.DMcalibration_slope)*(slope_weight_matrix*sample_aberration);

%generate DM command from the calulated zernike index
DMcommand_sample = -DM_zernike_index'*(diag(DM_calibration_factor)*Z2C);
DMcommand_sample_weight = -DM_zernike_index_weight'*(diag(DM_calibration_factor)*Z2C);
sample_index = DM_zernike_index.*DM_calibration_factor';% units in micron
sample_index_weight = DM_zernike_index_weight.*DM_calibration_factor';% units in micron;

%% send command to DM for full AO correction
% send non-weighted DM commands
hAOROIctrl.hDMctrl.senddata(DMcommand_sample+DMcommand_system_aberration);
% or send weighted DM commands
hAOROIctrl.hDMctrl.senddata(DMcommand_sample_weight+DMcommand_system_aberration);
