% Following the "DM_SHWS_calibration_control.m"
% Calibration SHWS with DM. 
% Adjust the parameters of the lenslet array.
% calculate the basis that calibrated with DM and SHWS

clear all;
close all;
clc;
%%
addpath('../examples/');
addpath('../data/');

%% set SHWS parameters
shstruct = struct();
shstruct.calibration_date = datestr(now, 'yyyymmddHHMMSS');
shstruct.use_bg = 0;               % subtract background from images
shstruct.use_gausscorrection = 1; %using gauss correction for DM calibration
shstruct.thresh_fine_img = 0.5;  % threshold for correlation image area detect
shstruct.npixsmall = 8;         % remove objects less than npixsmall
shstruct.npixsmall_corr = 9;      % remove objects less than npixsmall for corr-image
shstruct.strel_rad = 8;           % strel ratio (image processing in coarse grid)
shstruct.strel_rad_corr = 8;         % strel ratio (image processing in corr-image)
shstruct.coarse_grid_radius = round((500/13)/2)-1;  % radius for coarse grid
shstruct.percent = 0.8;            % threshold used in centroid fine grid
                                       % this value shoule be bigger when
                                       % more noisy of the image
                                       % scale estimated radius in the fine grid 
% define the size and sigma of gaussian filter used for psf
shstruct.psfsize_DM = 31;
shstruct.psfsize_sample = 31;
shstruct.psfsigma_sample = 1.5;%change from 1 to 1.5, want the osf bigger
                              % psfsigma value is associate with lensarray
                              % focal lenght
shstruct.psfsigma_DM = 1.5;%change from 1.5 to 1,
% this caused the fine grid to be smaller than the course grid
shstruct.multiply_est_radius = 0.9; %1/sqrt(2)
shstruct.centroid = @centroid;     % centroid algorithm
% lenslet documentation
shstruct.totlenses = 170;
shstruct.flength = 41.7e-3;
shstruct.pitch = 500e-6;
% camera documentation
shstruct.camera_pixsize = 13*1e-6;
shstruct.maxinteger = 2^16 - 1;       % maximum integer value for the image
% params Dai modal wavefront estimation
shstruct.sa_radius_m = shstruct.pitch/2;
shstruct.dai_n_zernike = 10;%5  % max radial order of the Zernike polynomials

%% pre-process of sh_flat
% load flat wf reference SHWS image
sh_flat=imread('ref_flat_wf.tif');
% aperture mask
imgtemp = int16(zeros(1024,1024));
xcenter = 512;
ycenter = 512;
imgrad = 268;
for x = 1:1024
    for y = 1:1024
        if sqrt((x-xcenter)^2+(y-ycenter)^2)<=imgrad
            imgtemp(y,x)=1;
        else
            imgtemp(y,x)=0;
        end
    end
end
sh_flat = sh_flat.*imgtemp;

%Gaussian correction to light intensity distribution 
gautemp = fspecial('gaussian',1024,imgrad);% the radius of the gauss temp can be smaller 
gautemp =gautemp.^2;
gautemp = (gautemp-min(min(gautemp)))./(max(max(gautemp))-min(min(gautemp)));
if shstruct.use_gausscorrection
    sh_flat = int16(double(sh_flat)./gautemp);
else
    sh_flat;
end

% generate threshould for course grid cal
shstruct.thresh_course_img  = graythresh(sh_flat);%this is crucial for grid generation

%% run the calibration
%shwfs_calibrate;

mkdir(sprintf('calibration-%s', shstruct.calibration_date));

% flat mirror reference
figure;
imshow(sh_flat);
imagesc(sh_flat);
title('reference image with Gaussian correction');
iimin = min(sh_flat(:));
iimax = max(sh_flat(:));
fprintf('[min, max] = [%.2f %.2f]\n', iimin, iimax);
shstruct.sh_flat = sh_flat;
shstruct.aperture_temp = imgtemp;
if exist('scflat', 'var')
    shstruct.scflat = scflat;
end


%% coarse grid with image processing
%load(filename_blank);
shstruct = shwfs_make_coarse_grid(shstruct);
filename_coarse = ...
    sprintf('calibration-%s/shstruct.coarse.mat', ...
    shstruct.calibration_date);
%ask_confirm('continue?');

save(filename_coarse, 'shstruct');
fprintf('saved %s\n', filename_coarse);
%% finer grid
load(filename_coarse);
shstruct = shwfs_make_fine_grid(shstruct);
filename_fine = ...
    sprintf('calibration-%s/shstruct.fine.mat', ...
    shstruct.calibration_date);
save(filename_fine, 'shstruct');
fprintf('saved %s\n', filename_fine);

%% make DM calibration slope matrix for basis
zernikeorder=60;
nspots = shstruct.nspots;
ref_cernter = shstruct.ord_centres;
h = waitbar(0,'DM calibration calculation...');
for i=1:zernikeorder
img = DM_calibration_shdata(:,:,i);
aberrated = shwfs_get_centres_nodraw(img, shstruct, shstruct.centroid);
slope(:,i) = reshape(aberrated,nspots*2,1) - reshape(ref_cernter,nspots*2,1);
waitbar(i/zernikeorder);
end
close(h);
Geninv = pinv(slope);
shstruct.DMcalibration_slope = slope;
shstruct.Geninv = Geninv;           

filename_fine = ...
    sprintf('calibration-%s/shstruct.mat', ...
    shstruct.calibration_date);
save(filename_fine, 'shstruct');
